@@ -1,4 +1,3 @@
-# $Id: POE.pm 2360 2008-06-26 05:25:26Z rcaputo $
 # Copyrights and documentation are after __END__.
 
 package POE;
@@ -6,21 +5,20 @@ package POE;
 use strict;
 use Carp qw( croak );
 
-use vars qw($VERSION $REVISION);
-$VERSION = '1.0003'; # NOTE - Should be #.#### (four decimal places)
-$REVISION = do {my($r)=(q$Revision: 2360 $=~/(\d+)/);sprintf"1.%04d",$r};
+use vars qw($VERSION);
+$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
 
 sub import {
   my $self = shift;
 
-  my @loops    = grep(/^Loop\:\:/, @_);
+  my @loops    = grep(/^(?:XS::)?Loop::/, @_);
   my @sessions = grep(/^(Session|NFA)$/, @_);
-  my @modules  = grep(!/^(Kernel|Session|NFA|Loop)$/, @_);
+  my @modules  = grep(!/^(Kernel|Session|NFA|(?:XS::)?Loop::[\w:]+)$/, @_);
 
   croak "can't use multiple event loops at once"
     if (@loops > 1);
   croak "POE::Session and POE::NFA export conflicting constants"
-    if grep(/^(Session|NFA)$/, @sessions) > 1;
+    if scalar @sessions > 1;
 
   # If a session was specified, use that.  Otherwise use Session.
   if (@sessions) {
@@ -72,7 +70,7 @@ __END__
 
 =head1 NAME
 
-POE - portable multitasking and networking framework for Perl
+POE - portable multitasking and networking framework for any event loop
 
 =head1 SYNOPSIS
 
@@ -117,27 +115,31 @@ POE is a framework for cooperative, event driven multitasking and
 networking in Perl.  Other languages have similar frameworks.  Python
 has Twisted.  TCL has "the event loop".
 
-POE originally was developed as the core of a persistent object server
-and runtime environment.  It has since evolved into something much
-more generic and widely useful.
-
 POE provides a unified interface for several other event loops,
-including select(), L<IO::Poll|IO::Poll>, L<Glib>, L<Gtk>, L<Tk>, L<Wx>, 
-L<Gtk2>, and so on.
-Check the CPAN for the full list of L<POE::Loop|POE::Loop> modules.
+including select(), L<IO::Poll|IO::Poll>, L<Glib>, L<Gtk>, L<Tk>,
+L<Wx>, and L<Gtk2>.  Many of these event loop interfaces were written
+by others, with the help of POE::Test::Loops.  They may be found on
+the CPAN.
 
-POE is designed in layers, each building atop the lower level ones.
-Programs are free to use POE at any level of abstraction, and
+POE achieves its its high degree of portability to different operating
+systems and Perl versions by being written entirely in Perl.  CPAN
+hosts optional XS modules for POE if speed is more desirable than
+portability.
+
+POE is designed in layers.  Each layer builds atop the lower level
+ones.  Programs are free to use POE at any level of abstraction, and
 different levels can be mixed and matched seamlessly within a single
-program.
+program.  Remember, though, that higher-level abstractions often
+require more resources than lower-level ones.  The conveniences they
+provide are not free.
 
 POE's bundled abstraction layers are the tip of a growing iceberg.
-L<Sprocket>, L<POE::Stage|POE::Stage>, and other CPAN distributions build upon this
-work.  You're encouraged to look around.
+L<Sprocket>, L<POE::Stage|POE::Stage>, and other CPAN distributions
+build upon this work.  You're encouraged to look around.
 
 No matter how high you go, though, it all boils down to calls to
-L<POE::Kernel|POE::Kernel>.  So your down-to-earth code can easily cooperate
-with stratospheric systems.
+L<POE::Kernel|POE::Kernel>.  So your down-to-earth code can easily
+cooperate with stratospheric systems.
 
 =head2 Layer 1: Kernel and Sessions
 
@@ -363,10 +365,9 @@ see the GETTING HELP section if you run into a problem.
 POE is expected to work on most forms of UNIX, including FreeBSD,
 MacOS X, Linux, Solaris.  Maybe even AIX and QNX, but we're not sure.
 
-POE has also been tested on Windows XP, using the latest version of
-ActiveState Perl and Cygwin Perl.  Support for the Neopolitan Perls
-(Vanilla, Strawberry and Chocolate) is still early, but there should
-be no major show stoppers.
+POE is also tested on Windows XP, using the latest version of
+ActiveState, Strawberry and Cygwin Perl.  POE is fully supported with
+Strawberry Perl, as it's included in the Strawberry distribution.
 
 OS/2 and MacOS 9 have been reported to work in the past, but nobody
 seems to be testing there anymore.  Reports and patches are still
@@ -382,7 +383,7 @@ POE's quality is due in large part to the fine work of Chris Williams
 and the other CPAN testers.  They have dedicated resources towards
 ensuring CPAN distributions pass their own tests, and we watch their
 reports religiously.  You can, too.  The latest POE test reports can
-be found at <http://testers.cpan.org/search?request=dist&dist=POE>.
+be found at L<http://cpantesters.org/distro/P/POE.html>.
 
 Thanks also go out to Benjamin Smith and the 2006 Google Summer of
 Code.  Ben was awarded a grant to improve POE's test suite, which he
@@ -462,10 +463,6 @@ an e-mail via his CPAN address.
 
 Broken down by abstraction layer.
 
-TODO - Link to web resources.
-
-TODO - Verify this is complete.
-
 =head2 Layer 1
 
 L<POE::Kernel>, L<POE::Session>, L<POE::NFA>
@@ -504,18 +501,21 @@ L<POE::Resource::Sessions>, L<POE::Resource::Signals>
 
 L<POE::Pipe>, L<POE::Pipe::OneWay>, L<POE::Pipe::TwoWay>
 
-=head1 BUGS
+=head2 Home Page
 
-The tests only cover about 70% of POE.  Getting higher coverage is
-really hard.
+http://poe.perl.org/
 
-Bug reports, suggestions, and feedback of all kinds should be e-mailed
-to <bug-POE@rt.cpan.org>.  They will be entered into our request queue
-and remain there until addressed.  If your return address is valid,
-you'll be notified when the status of your request changes.
+=head2 Bug Tracker
 
-Outstanding issues, including wish list items, are browsable in POE's
-RT queue at L<http://rt.cpan.org/>.
+https://rt.cpan.org/Dist/Display.html?Status=Active&Queue=POE
+
+=head2 Repository
+
+https://poe.svn.sourceforge.net/svnroot/poe/trunk/poe
+
+=head2 Other Resources
+
+http://search.cpan.org/dist/POE/
 
 =head1 AUTHORS & COPYRIGHT
 
@@ -590,6 +590,9 @@ support different serialization methods.  He has also improved POE's quality
 by finding and fixing several bugs.  He provided POE a much needed code
 review around version 0.06.
 
+Lately, Philip tracked down the race condition in signal handling and
+fixed it with the signal pipe.
+
 =item Arnar M. Hrafnkelsson
 
 Arnar is <addi@umich.edu>.  Addi tested POE and L<POE::Component::IRC|POE::Component::IRC> on
@@ -672,7 +675,7 @@ to thank you for your interest, and he has more thanks than he can
 count for all the people who have contributed.  POE would not be
 nearly as cool without you.
 
-Except where otherwise noted, POE is Copyright 1998-2007 Rocco Caputo.
+Except where otherwise noted, POE is Copyright 1998-2009 Rocco Caputo.
 All rights reserved.  POE is free software; you may redistribute it
 and/or modify it under the same terms as Perl itself.
 
@@ -683,3 +686,4 @@ Thank you for reading!
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
