@@ -1,11 +1,9 @@
-# $Id: TCP.pm 2505 2009-03-12 00:54:24Z apocal $
-
 package POE::Component::Server::TCP;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2505 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
@@ -36,9 +34,6 @@ sub new {
   croak "$mi requires an even number of parameters" if (@_ & 1);
   my %param = @_;
 
-  # Validate what we're given.
-  croak "$mi needs a Port parameter" unless exists $param{Port};
-
   # Extract parameters.
   my $alias   = delete $param{Alias};
   my $address = delete $param{Address};
@@ -47,9 +42,12 @@ sub new {
   my $domain  = delete($param{Domain}) || AF_INET;
   my $concurrency = delete $param{Concurrency};
 
+  $port = 0 unless defined $port;
+
   foreach (
     qw(
-      Acceptor Error ClientInput ClientConnected ClientDisconnected
+      Acceptor Error ClientInput
+      ClientPreConnect ClientConnected ClientDisconnected
       ClientError ClientFlushed
       ClientLow ClientHigh
     )
@@ -90,6 +88,7 @@ sub new {
     }
   }
 
+  my $client_pre_connect  = delete $param{ClientPreConnect};
   my $client_connected    = delete $param{ClientConnected};
   my $client_disconnected = delete $param{ClientDisconnected};
   my $client_error        = delete $param{ClientError};
@@ -100,6 +99,11 @@ sub new {
   my $session_type        = delete $param{SessionType};
   my $session_params      = delete $param{SessionParams};
   my $server_started      = delete $param{Started};
+  my $listener_args       = delete $param{ListenerArgs};
+
+  $listener_args = [] unless defined $listener_args;
+  croak "ListenerArgs must be an array reference"
+    unless ref($listener_args) eq 'ARRAY';
 
   if (exists $param{Args}) {
     if (exists $param{ClientArgs}) {
@@ -111,7 +115,7 @@ sub new {
     }
   }
 
-  my $args                = delete($param{ClientArgs}) || delete($param{Args});
+  my $client_args = delete($param{ClientArgs}) || delete($param{Args});
 
   # Defaults.
 
@@ -124,7 +128,6 @@ sub new {
   $address = INADDR_ANY unless defined $address;
 
   $error_callback = \&_default_server_error unless defined $error_callback;
-  $server_started = sub {} unless ref($server_started) eq 'CODE';
 
   $session_type = 'POE::Session' unless defined $session_type;
   if (defined($session_params) && ref($session_params)) {
@@ -137,10 +140,7 @@ sub new {
 
   if (defined $client_input) {
     $client_error  = \&_default_client_error unless defined $client_error;
-    $client_connected    = sub {} unless defined $client_connected;
-    $client_disconnected = sub {} unless defined $client_disconnected;
-    $client_flushed      = sub {} unless defined $client_flushed;
-    $args = [] unless defined $args;
+    $client_args         = []     unless defined $client_args;
 
     # Extra states.
 
@@ -168,7 +168,7 @@ sub new {
       unless ref($object_states) eq 'ARRAY';
 
     croak "ClientArgs must be an array reference"
-      unless ref($args) eq 'ARRAY';
+      unless ref($client_args) eq 'ARRAY';
 
     # Sanity check, thanks to crab@irc for making this mistake, ha!
     # TODO we could move this to POE::Session and make it a
@@ -239,7 +239,6 @@ sub new {
       $accept_callback = sub {
         my ($socket, $remote_addr, $remote_port) = @_[ARG0, ARG1, ARG2];
 
-
         $session_type->create(
           @$session_params,
           inline_states => {
@@ -271,8 +270,18 @@ sub new {
 
               $heap->{remote_port} = $remote_port;
 
+              my $socket = $_[ARG0];
+              if ($client_pre_connect) {
+                $socket = $client_pre_connect->(@_);
+                unless (fileno($socket)) {
+                  # TODO - Error callback?  Disconnected callback?
+                  # TODO - Should we do this before starting the child?
+                  return;
+                }
+              }
+
               $heap->{client} = POE::Wheel::ReadWrite->new(
-                Handle       => splice(@_, ARG0, 1),
+                Handle       => $socket,
                 Driver       => POE::Driver::SysRW->new(),
                 _get_filters(
                   $client_filter,
@@ -295,7 +304,11 @@ sub new {
                 ),
               );
 
-              $client_connected->(@_);
+              # Expand the Args constructor array, and place a copy
+              # into @_[ARG0..].  There are only 2 parameters.
+              splice(@_, ARG0, 2, @{$_[ARG1]});
+
+              $client_connected and $client_connected->(@_);
             },
             tcp_server_got_high => $high_event,
             tcp_server_got_low => $low_event,
@@ -323,10 +336,10 @@ sub new {
             tcp_server_got_flush => sub {
               my $heap = $_[HEAP];
               DEBUG and warn "$$: $alias child Flush";
-              $client_flushed->(@_);
+              $client_flushed and $client_flushed->(@_);
               if ($heap->{shutdown}) {
                 DEBUG and warn "$$: $alias child Flush, callback";
-                $client_disconnected->(@_);
+                $client_disconnected and $client_disconnected->(@_);
                 delete $heap->{client};
               }
             },
@@ -340,7 +353,7 @@ sub new {
                   not $heap->{client}->get_driver_out_octets()
                 ) {
                   DEBUG and warn "$$: $alias child Shutdown, callback";
-                  $client_disconnected->(@_);
+                  $client_disconnected and $client_disconnected->(@_);
                   delete $heap->{client};
                 }
               }
@@ -373,7 +386,9 @@ sub new {
           package_states => $package_states,
           object_states  => $object_states,
 
-          args => [ $socket, $args ],
+          # XXX - If you change the number of args here, also change
+          # the splice elsewhere.
+          args => [ $socket, $client_args ],
         );
       };
     }
@@ -431,7 +446,7 @@ sub new {
           SuccessEvent => 'tcp_server_got_connection',
           FailureEvent => 'tcp_server_got_error',
         );
-        $server_started->(@_);
+        $server_started and $server_started->(@_);
       },
       # Catch an error.
       tcp_server_got_error => $error_callback,
@@ -504,7 +519,7 @@ sub new {
       _child  => sub { },
     },
 
-    args => $args,
+    args => $listener_args,
   )->ID;
 
   # Return the session ID.
@@ -569,7 +584,7 @@ sub _test_filter {
     if (!$eval and $@) {
         carp(
           "Failed to load [$filter]\n" .
-          "Reason $@\nUsing defualt POE::Filter::Line "
+          "Reason $@\nUsing default POE::Filter::Line "
         );
         return 0;
     }
@@ -666,12 +681,27 @@ POE::Wheel::SocketFactory's C<SuccessEvent>.
 Otherwise, the default C<Acceptor> callback will start a new session
 to handle each connection.  These child sessions do not have their own
 aliases, but their C<ClientConnected> and C<ClientDisconnected>
-callbacks may register and unregister the sessions with a shared
-namespace of their own.
+callbacks may be used to register and unregister the sessions with a
+shared namespace, such as a hash keyed on session IDs, or an object
+that manages such a hash.
+
+  my %client_namespace;
+
+  sub handle_client_connected {
+    my $client_session_id = $_[SESSION]->ID;
+    $client_namespace{$client_session_id} = \%anything;
+  }
+
+  sub handle_client_disconnected {
+    my $client_session_id = $_[SESSION]->ID;
+    $client_namespace{$client_session_id} = \%anything;
+  }
 
 The component's C<Started> callback is invoked at the end of the
-master session's startup routine.  This callback's parameters are
-the usual ones for C<_start>.
+master session's start-up routine.  The @_[ARG0..$#_] parameters are
+set to a copy of the values in the server's C<ListenerArgs>
+constructor parameter.  The other parameters are standard for
+POE::Session's _start handlers.
 
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
@@ -700,12 +730,41 @@ The component's C<ClientInput> callback defines how child sessions
 will handle input from their clients.  Its parameters are that of
 POE::Wheel::ReadWrite's C<InputEvent>.
 
-C<ClientConnected> is called at the end of the child session's
-C<_start> routine.  In addition to the usual C<_start> parameters, it
-includes the socket in $_[ARG0] and the contents of the component's
-C<Args> constructor parameter in $_[ARG1].
+As mentioned C<ClientConnected> is called at the end of the child
+session's C<_start> routine.  The C<ClientConneted> callback receives
+the same parameters as the client session's _start does.  The arrayref
+passed to the constructor's C<Args> parameter is flattened and
+included in C<ClientConnected>'s parameters as @_[ARG0..$#_].
 
-TODO - Should C<Args> be flattened into C<ARG1..$%_>?
+  sub handle_client_connected {
+    my @constructor_args = @_[ARG0..$#_];
+    ...
+  }
+
+C<ClientPreConnect> is called before C<ClientConnected>, and it has
+different parameters: $_[ARG0] contains a copy of the client socket
+before it's given to POE::Wheel::ReadWrite for management.  Most HEAP
+members are set, except of course $_[HEAP]{client}, because the
+POE::Wheel::ReadWrite has not yet been created yet.
+C<ClientPreConnect> may enable SSL on the socket, using
+POE::Component::SSLify.  C<ClientPreConnect> must return a valid
+socket to complete the connection; the client will be disconnected if
+anything else is returned.
+
+  sub handle_client_pre_connect {
+
+    # Make sure the remote address and port are valid.
+    return undef unless validate(
+      $_[HEAP]{remote_ip}, $_[HEAP]{remote_port}
+    );
+
+    # SSLify the socket, which is in $_[ARG0].
+    my $socket = eval { Server_SSLify($_[ARG0]) };
+    return undef if $@;
+
+    # Return the SSL-ified socket.
+    return $socket;
+  }
 
 C<ClientDisconnected> is called when the client has disconnected,
 either because the remote socket endpoint has closed or the local
@@ -754,7 +813,7 @@ C<Alias> parameter.
 
 POE::Component::Server::TCP does a lot of work in its constructor.
 The design goal is to push as much overhead into one-time construction
-so that ongoing runtime has less overhead.  Because of this, the
+so that ongoing run-time has less overhead.  Because of this, the
 server's constructor can take quite a daunting number of parameters.
 
 POE::Component::Server::TCP always returns a POE::Session ID for the
@@ -897,8 +956,8 @@ transactions.
 C<Hostname> is the optional non-packed name of the interface the TCP
 server will bind to.  The hostname will always be resolved via
 inet_aton() and so can either be a dotted quad or a name.  Name
-resolution is a one-time startup action; there are no ongoing runtime
-penalties for using it.
+resolution is a one-time start-up action; there are no ongoing
+run-time penalties for using it.
 
 C<Hostname> guarantees name resolution, where C<Address> does not.
 It's therefore preferred to use C<Hostname> in cases where resolution
@@ -910,8 +969,8 @@ C<InlineStates> is optional.  If specified, it must hold a hashref of
 named callbacks.  Its syntax is that of POE:Session->create()'s
 inline_states parameter.
 
-Remember: These InlineStates handlers will be added to the main
-listening session, not to every connection.  A yield() in a connection
+Remember: These InlineStates handlers will be added to the client
+sessions, not to the main listening session.  A yield() in the listener
 will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
@@ -924,8 +983,8 @@ If C<ObjectStates> is specified, it must holde an arrayref of objects
 and the events they will handle.  The arrayref must follow the syntax
 for POE::Session->create()'s object_states parameter.
 
-Remember: These ObjectStates handlers will be added to the main
-listening session, not to every connection.  A yield() in a connection
+Remember: These ObjectStates handlers will be added to the client 
+sessions, not to the main listening session.  A yield() in the listener
 will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
@@ -939,8 +998,8 @@ package names and the events they will handle  The arrayref must
 follow the syntax for POE::Session->create()'s package_states
 parameter.
 
-Remember: These PackageStates handlers will be added to the main
-listening session, not to every connection.  A yield() in a connection
+Remember: These PackageStates handlers will be added to the client 
+sessions, not to the main listening session.  A yield() in the listener
 will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
@@ -950,7 +1009,7 @@ callback if you want to specify your own "_start" event.
 =head4 Port
 
 C<Port> contains the port the listening socket will be bound to.  It
-defaults to INADDR_ANY, which usually lets the operating system pick a
+defaults to 0, which usually lets the operating system pick a
 port at random.
 
   Port => 30023
@@ -964,6 +1023,12 @@ main server session's context.  It notifies the server that it has
 fully started.  The callback's parameters are the usual for a
 session's _start handler.
 
+=head4 ListenerArgs
+
+C<ListenerArgs> is passed to the listener session as the C<args> parameter.  In
+other words, it must be an arrayref, and the values are are passed into the
+C<Started> handler as ARG0, ARG1, etc.
+
 =head3 Connection Session Configuration
 
 These constructor parameters affect the individual sessions that
@@ -972,9 +1037,8 @@ interact with established connections.
 =head4 ClientArgs
 
 C<ClientArgs> is optional.  When specified, it holds an ARRAYREF that
-will be passed to the C<ClientConnected> callback in $_[ARG1].
-(ClientConnected's $_[ARG0] contains the newly accepted client
-socket.)
+will be expanded one level and passed to the C<ClientConnected>
+callback in @_[ARG0..$#_].
 
 =head4 ClientConnected
 
@@ -983,11 +1047,11 @@ C<ClientConnected> is a callback that notifies the application when a
 client's session is started and ready for operation.  Banners are
 often sent to the remote client from this callback.
 
-C<ClientConnected> callbacks receive the usual POE parameters plus:
-The newly accepted client socket in $_[ARG0] and the ARRAYREF
-specified in C<ClientArgs> in $_[ARG1].
+The @_[ARG0..$#_] parameters to C<ClientConnected> are a copy of the
+values in the C<ClientArgs> constructor parameter's array reference.
+The other @_ members are standard for a POE::Session _start handler.
 
-C<ClientConnected> is called once per session startup.  It will never
+C<ClientConnected> is called once per session start-up.  It will never
 be called twice for the same connection.
 
   ClientConnected => sub {
@@ -995,8 +1059,6 @@ be called twice for the same connection.
     # Other client initialization here.
   },
 
-TODO - Show Args and client socket introspection.
-
 =head4 ClientDisconnected
 
 C<ClientDisconnected> is a callback that will be invoked when the
@@ -1044,7 +1106,7 @@ POE::Filter class to use.  Each new connection will be given a freshly
 instantiated filter of that class.  No constructor parameters will be
 passed.
 
-  ClientFIlter => "POE::Filter::Stream",
+  ClientFilter => "POE::Filter::Stream",
 
 Some filters require constructor parameters.  These may be specified
 by an ARRAYREF.  The first element is the POE::Filter class name, and
@@ -1093,7 +1155,7 @@ flushed before finishing a client shutdown.
 C<ClientInput> defines a per-connection callback to handle client
 input.  This callback receives its parameters directly from
 POE::Wheel::ReadWrite's C<InputEvent>.  ARG0 contains the input
-recored, the format of which is defined by C<ClientFilter> or
+record, the format of which is defined by C<ClientFilter> or
 C<ClientInputFilter>.  ARG1 has the wheel's unique ID, and so on.
 Please see POE:Wheel::ReadWrite for an in-depth description of
 C<InputEvent>.
@@ -1270,7 +1332,7 @@ procedure.
 
 =head3 remote_ip
 
-$_[HEAP]{remote_ip} contains the remote client's nuermic address in
+$_[HEAP]{remote_ip} contains the remote client's numeric address in
 human-readable form.
 
 =head3 remote_port
@@ -1281,7 +1343,7 @@ in human-readable form.
 =head3 remote_addr
 
 $_[HEAP]{remote_addr} contains the remote client's packed socket
-address in computer-readabe form.
+address in computer-readable form.
 
 =head3 shutdown
 
@@ -1321,8 +1383,6 @@ Some use cases require different session classes for the listener and
 the connection handlers.  This isn't currently supported.  Please send
 patches. :)
 
-TODO - Rename C<Args> into C<ClientArgs>.
-
 TODO - Document that Reuse is set implicitly.
 
 =head1 AUTHORS & COPYRIGHTS
