@@ -1,11 +1,9 @@
-# $Id: Driver.pm 2331 2008-05-26 01:19:11Z rcaputo $
-
 package POE::Driver;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2331 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(croak);
 
@@ -102,7 +100,7 @@ internally.
 =item put ARRAYREF
 
 put() accepts an ARRAYREF of raw octet chunks.  These octets are added
-to the dirver's internal output queue or buffer.  put() returns the
+to the driver's internal output queue or buffer.  put() returns the
 number of octets pending output after the new octets are buffered.
 
 Some drivers may flush data immediately from their put() methods.
@@ -110,7 +108,7 @@ Some drivers may flush data immediately from their put() methods.
 =item flush FILEHANDLE
 
 flush() attempts to write a driver's buffered data to a given
-FIELHANDLE.  The driver should flush as much data as possible in a
+FILEHANDLE.  The driver should flush as much data as possible in a
 single flush() call.
 
 flush() returns the number of octets remaining in the driver's output
@@ -166,3 +164,4 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
