@@ -1,13 +1,11 @@
-# $Id: Extrefs.pm 1903 2006-03-20 04:44:08Z rcaputo $
-
 # The data necessary to manage tagged extra/external reference counts
 # on sessions, and the accessors to get at them sanely from other
 # files.
 
-package POE::Resources::Extrefs;
+package POE::Resource::Extrefs;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -42,14 +40,21 @@ sub _data_extref_finalize {
 # time the tag is used in the session, then increment the session's
 # reference count as well.  Returns the tag's new reference count.
 #
-# -><- Allows incrementing reference counts on sessions that don't
+# TODO Allows incrementing reference counts on sessions that don't
 # exist, but the public interface catches that.
+#
+# TODO Need to track extref ownership for signal-based session
+# termination.  One problem seen is that signals terminate sessions
+# out of order.  Owners think extra refcounts exist for sessions that
+# are no longer around.  Ownership trees give us a few benefits: We
+# can make sure sessions destruct in a cleaner order.  We can detect
+# refcount loops and possibly prevent that.
 
 sub _data_extref_inc {
   my ($self, $session, $tag) = @_;
   my $refcount = ++$kr_extra_refs{$session}->{$tag};
 
-  # -><- We could probably get away with only incrementing the
+  # TODO We could probably get away with only incrementing the
   # session's master refcount once, as long as any extra refcount is
   # positive.  Then the session reference count would be a flag
   # instead of a counter.
@@ -69,7 +74,7 @@ sub _data_extref_inc {
 # if the count reaches zero.  Return the new reference count or undef
 # if the tag doesn't exist.
 #
-# -><- Allows negative reference counts, and the resulting hilarity.
+# TODO Allows negative reference counts, and the resulting hilarity.
 # Hopefully the public interface won't allow it.
 
 sub _data_extref_dec {
@@ -167,32 +172,41 @@ __END__
 
 =head1 NAME
 
-POE::Resources::Extrefs - tagged "extra" ref. count management for POE::Kernel
+POE::Resource::Extrefs - internal reference counts manager for POE::Kernel
 
 =head1 SYNOPSIS
 
-Used internally by POE::Kernel.  Better documentation will be
-forthcoming.
+There is no public API.
 
 =head1 DESCRIPTION
 
-This module encapsulates and provides accessors for POE::Kernel's data
-structures that manage tagged reference counts.  It is used internally
-by POE::Kernel and has no public interface.
+POE::Resource::Extrefs is a mix-in class for POE::Kernel.  It provides
+the features to manage session reference counts, specifically the ones
+that applications may use.  POE::Resource::Extrefs is used internally
+by POE::Kernel, so it has no public interface.
 
 =head1 SEE ALSO
 
-See L<POE::Kernel> for documentation on tagged reference counts.
+See L<POE::Kernel/Public Reference Counters> for the public extref
+API.
+
+See L<POE::Kernel/Resources> for for public information about POE
+resources.
+
+See L<POE::Resource> for general discussion about resources and the
+classes that manage them.
 
 =head1 BUGS
 
-There is no mechanism in place to prevent extra reference count names
-from clashing.
-
-Probably others.
+Reference counters have no ownership information, so one entity's
+reference counts may conflict with another's.  This is usually not a
+problem if all entities behave.
 
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
