@@ -1,27 +1,37 @@
-# $Id: FileHandles.pm 1903 2006-03-20 04:44:08Z rcaputo $
-
 # Manage file handles, associated descriptors, and read/write modes
 # thereon.
 
-package POE::Resources::FileHandles;
+package POE::Resource::FileHandles;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 1903 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
 
 use strict;
 
+use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
+use IO::Handle ();
+use FileHandle ();
+
 ### Some portability things.
 
 # Provide dummy constants so things at least compile.  These constants
 # aren't used if we're RUNNING_IN_HELL, but Perl needs to see them.
 
 BEGIN {
-  if (RUNNING_IN_HELL) {
-    eval '*F_GETFL = sub { 0 };';
-    eval '*F_SETFL = sub { 0 };';
+  # older perls than 5.10 needs a kick in the arse to AUTOLOAD the constant...
+  eval "F_GETFL" if $] < 5.010;
+
+  if ( ! defined &Fcntl::F_GETFL ) {
+    if ( ! defined prototype "F_GETFL" ) {
+      *F_GETFL = sub { 0 };
+      *F_SETFL = sub { 0 };
+    } else {
+      *F_GETFL = sub () { 0 };
+      *F_SETFL = sub () { 0 };
+    }
   }
 }
 
@@ -35,14 +45,13 @@ my $kr_queue;
 ### more unique.
 
 my %kr_filenos;
+BEGIN { $poe_kernel->[KR_FILENOS] = \%kr_filenos; }
 
 sub FNO_MODE_RD      () { MODE_RD } # [ [ (fileno read mode structure)
 # --- BEGIN SUB STRUCT 1 ---        #
 sub FMO_REFCOUNT     () { 0      }  #     $fileno_total_use_count,
 sub FMO_ST_ACTUAL    () { 1      }  #     $requested_file_state (see HS_PAUSED)
-sub FMO_ST_REQUEST   () { 2      }  #     $actual_file_state (see HS_PAUSED)
-sub FMO_EV_COUNT     () { 3      }  #     $number_of_pending_events,
-sub FMO_SESSIONS     () { 4      }  #     { $session_watching_this_handle =>
+sub FMO_SESSIONS     () { 2      }  #     { $session_watching_this_handle =>
                                     #       { $handle_watched_as =>
 # --- BEGIN SUB STRUCT 2 ---        #
 sub HSS_HANDLE       () { 0      }  #         [ $blessed_handle,
@@ -63,7 +72,7 @@ sub FNO_MODE_EX      () { MODE_EX } #   [ (expedite mode struct is the same)
 sub FNO_TOT_REFCOUNT () { 3      }  #   $total_number_of_file_watchers,
                                     # ]
 
-### These are the values for FMO_ST_ACTUAL and FMO_ST_REQUEST.
+### These are the values for FMO_ST_ACTUAL.
 
 sub HS_STOPPED   () { 0x00 }   # The file has stopped generating events.
 sub HS_PAUSED    () { 0x01 }   # The file temporarily stopped making events.
@@ -87,11 +96,6 @@ sub SH_MODECOUNT  () {  2 } #          [ $read_reference_count,     (MODE_RD)
                             #      },
                             #    },
 
-sub _data_handle_preload {
-    $poe_kernel->[KR_FILENOS] = \%kr_filenos;
-}
-use POE::API::ResLoader \&_data_handle_preload;
-
 ### Begin-run initialization.
 
 sub _data_handle_initialize {
@@ -113,7 +117,6 @@ sub _data_handle_finalize {
     _warn(
       "!!!\tRead:\n",
       "!!!\t\trefcnt  = $rd->[FMO_REFCOUNT]\n",
-      "!!!\t\tev cnt  = $rd->[FMO_EV_COUNT]\n",
     );
     while (my ($ses, $ses_rec) = each(%{$rd->[FMO_SESSIONS]})) {
       _warn "!!!\t\tsession = $ses\n";
@@ -130,7 +133,6 @@ sub _data_handle_finalize {
     _warn(
       "!!!\tWrite:\n",
       "!!!\t\trefcnt  = $wr->[FMO_REFCOUNT]\n",
-      "!!!\t\tev cnt  = $wr->[FMO_EV_COUNT]\n",
     );
     while (my ($ses, $ses_rec) = each(%{$wr->[FMO_SESSIONS]})) {
       _warn "!!!\t\tsession = $ses\n";
@@ -147,7 +149,6 @@ sub _data_handle_finalize {
     _warn(
       "!!!\tException:\n",
       "!!!\t\trefcnt  = $ex->[FMO_REFCOUNT]\n",
-      "!!!\t\tev cnt  = $ex->[FMO_EV_COUNT]\n",
     );
     while (my ($ses, $ses_rec) = each(%{$ex->[FMO_SESSIONS]})) {
       _warn "!!!\t\tsession = $ses\n";
@@ -178,99 +179,53 @@ sub _data_handle_finalize {
   return $finalized_ok;
 }
 
-### Ensure a handle's actual state matches its requested one.  Pause
-### or resume the handle as necessary.
-
-sub _data_handle_resume_requested_state {
-  my ($self, $handle, $mode) = @_;
-  my $fileno = fileno($handle);
-
-  # Skip the rest if we aren't watching the file descriptor.  This
-  # seems like a kludge: should we even be called if the descriptor
-  # isn't watched?
-  return unless exists $kr_filenos{$fileno};
-
-  my $kr_fno_rec  = $kr_filenos{$fileno}->[$mode];
-
-  if (TRACE_FILES) {
-    _warn(
-      "<fh> decrementing event count in mode ($mode) ",
-      "for fileno (", $fileno, ") from count (",
-      $kr_fno_rec->[FMO_EV_COUNT], ")"
-    );
-  }
-
-  # If all events for the fileno/mode pair have been delivered, then
-  # resume the filehandle's watcher.  This decrements FMO_EV_COUNT
-  # because the event has just been dispatched.  This makes sense.
-
-  unless (--$kr_fno_rec->[FMO_EV_COUNT]) {
-    if ($kr_fno_rec->[FMO_ST_REQUEST] & HS_PAUSED) {
-      $self->loop_pause_filehandle($handle, $mode);
-      $kr_fno_rec->[FMO_ST_ACTUAL] = HS_PAUSED;
-    }
-    elsif ($kr_fno_rec->[FMO_ST_REQUEST] & HS_RUNNING) {
-      $self->loop_resume_filehandle($handle, $mode);
-      $kr_fno_rec->[FMO_ST_ACTUAL] = HS_RUNNING;
-    }
-    elsif (ASSERT_DATA) {
-      _trap();
-    }
-  }
-  elsif (ASSERT_DATA) {
-    if ($kr_fno_rec->[FMO_EV_COUNT] < 0) {
-      _trap "handle event count went below zero";
-    }
-  }
-}
-
 ### Enqueue "select" events for a list of file descriptors in a given
 ### access mode.
 
 sub _data_handle_enqueue_ready {
-  my ($self, $mode, @filenos) = @_;
+  my ($self, $mode) = splice(@_, 0, 2);
 
-  foreach my $fileno (@filenos) {
+  my $now = time();
+  foreach my $fileno (@_) {
     if (ASSERT_DATA) {
       _trap "internal inconsistency: undefined fileno" unless defined $fileno;
     }
 
-    my $kr_fno_rec = $kr_filenos{$fileno}->[$mode];
+    # By-pass the event queue for things that come over the pipe:
+    # this reduces signal latency
+    if( USE_SIGNAL_PIPE ) {
+      # _warn "fileno=$fileno signal_pipe_read=$POE::Kernel::signal_pipe_read_fd";
+      if( $fileno == $POE::Kernel::signal_pipe_read_fd ) {
+        $self->_data_sig_pipe_read( $fileno, $mode );
+        next;
+      }
+    }
 
-    # Gather all the events to emit for this fileno/mode pair.
+    # Avoid autoviviying an empty $kr_filenos record if the fileno has
+    # been deactivated.  This can happen if a file descriptor is ready
+    # in multiple modes, and an earlier dispatch removes it before a
+    # later dispatch happens.
+    next unless exists $kr_filenos{$fileno};
 
-    my @selects = map { values %$_ } values %{ $kr_fno_rec->[FMO_SESSIONS] };
+    # Gather and dispatch all the events for this fileno/mode pair.
 
-    # Emit them.
-
-    foreach my $select (@selects) {
-      $self->_data_ev_enqueue(
+    foreach my $select (
+      map { values %$_ }
+      values %{ $kr_filenos{$fileno}[$mode][FMO_SESSIONS] }
+    ) {
+      $self->_dispatch_event(
         $select->[HSS_SESSION], $select->[HSS_SESSION],
-        $select->[HSS_STATE], ET_SELECT,
-        [ $select->[HSS_HANDLE],  # EA_SEL_HANDLE
+        $select->[HSS_STATE], ET_SELECT, [
+          $select->[HSS_HANDLE],  # EA_SEL_HANDLE
           $mode,                  # EA_SEL_MODE
           @{$select->[HSS_ARGS]}, # EA_SEL_ARGS
         ],
-        __FILE__, __LINE__, undef, time(),
+        __FILE__, __LINE__, undef, $now, -__LINE__
       );
-
-      # Count the enqueued event.  This increments FMO_EV_COUNT
-      # because an event has just been enqueued.  This makes sense.
-
-      unless ($kr_fno_rec->[FMO_EV_COUNT]++) {
-        my $handle = $select->[HSS_HANDLE];
-        $self->loop_pause_filehandle($handle, $mode);
-        $kr_fno_rec->[FMO_ST_ACTUAL] = HS_PAUSED;
-      }
-
-      if (TRACE_FILES) {
-        _warn(
-          "<fh> incremented event count in mode ($mode) ",
-          "for fileno ($fileno) to count ($kr_fno_rec->[FMO_EV_COUNT])"
-        );
-      }
     }
   }
+
+  $self->_data_ses_gc_sweep();
 }
 
 ### Test whether POE is tracking a file handle.
@@ -302,65 +257,24 @@ sub _data_handle_add {
     $kr_filenos{$fd} =
       [ [ 0,          # FMO_REFCOUNT    MODE_RD
           HS_PAUSED,  # FMO_ST_ACTUAL
-          HS_PAUSED,  # FMO_ST_REQUEST
-          0,          # FMO_EV_COUNT
           { },        # FMO_SESSIONS
         ],
         [ 0,          # FMO_REFCOUNT    MODE_WR
           HS_PAUSED,  # FMO_ST_ACTUAL
-          HS_PAUSED,  # FMO_ST_REQUEST
-          0,          # FMO_EV_COUNT
           { },        # FMO_SESSIONS
         ],
         [ 0,          # FMO_REFCOUNT    MODE_EX
           HS_PAUSED,  # FMO_ST_ACTUAL
-          HS_PAUSED,  # FMO_ST_REQUEST
-          0,          # FMO_EV_COUNT
           { },        # FMO_SESSIONS
         ],
         0,            # FNO_TOT_REFCOUNT
       ];
 
     if (TRACE_FILES) {
-      _warn "<fh> adding fd ($fd) in mode ($mode)";
+      _warn "<fh> adding $handle fd ($fd) in mode ($mode)";
     }
 
-    # For DOSISH systems like OS/2.  Wrapped in eval{} in case it's a
-    # tied handle that doesn't support binmode.
-    eval { binmode *$handle };
-
-    # Turn off blocking unless it's tied or a plain file.
-    unless (tied *$handle or -f $handle) {
-
-      unless (RUNNING_IN_HELL) {
-        if ($] >= 5.008) {
-          $handle->blocking(0);
-        }
-        else {
-          # Long, drawn out, POSIX way.
-          my $flags = fcntl($handle, F_GETFL, 0)
-            or _trap "fcntl($handle, F_GETFL, etc.) fails: $!\n";
-          until (fcntl($handle, F_SETFL, $flags | O_NONBLOCK)) {
-            _trap "fcntl($handle, FSETFL, etc) fails: $!"
-              unless $! == EAGAIN or $! == EWOULDBLOCK;
-          }
-        }
-      }
-      else {
-        # Do it the Win32 way.
-        my $set_it = "1";
-
-        # 126 is FIONBIO (some docs say 0x7F << 16)
-        ioctl(
-          $handle,
-          0x80000000 | (4 << 16) | (ord('f') << 8) | 126,
-          \$set_it
-        ) or _trap "ioctl($handle, FIONBIO, $set_it) fails: $!\n";
-      }
-    }
-
-    # Turn off buffering.
-    CORE::select((CORE::select($handle), $| = 1)[0]);
+    $self->_data_handle_condition( $handle );
   }
 
   # Cache some high-level lookups.
@@ -376,16 +290,10 @@ sub _data_handle_add {
 
     if (exists $kr_fno_rec->[FMO_SESSIONS]->{$session}->{$handle}) {
       if (TRACE_FILES) {
-        _warn(
-          "<fh> running fileno($fd) mode($mode) " .
-          "count($kr_fno_rec->[FMO_EV_COUNT])"
-        );
+        _warn("<fh> running $handle fileno($fd) mode($mode)");
       }
-      unless ($kr_fno_rec->[FMO_EV_COUNT]) {
-        $self->loop_resume_filehandle($handle, $mode);
-        $kr_fno_rec->[FMO_ST_ACTUAL] = HS_RUNNING;
-      }
-      $kr_fno_rec->[FMO_ST_REQUEST] = HS_RUNNING;
+      $self->loop_resume_filehandle($handle, $mode);
+      $kr_fno_rec->[FMO_ST_ACTUAL] = HS_RUNNING;
     }
 
     # The session is watching it by a different handle.  It can't be
@@ -408,18 +316,45 @@ sub _data_handle_add {
             $why = "closed";
           }
           elsif (fileno($handle) == fileno($other_handle)) {
-            $why = "still open";
+            $why = "open";
           }
           else {
-            $why = "open as different fd";
+            $why = "open with different file descriptor";
           }
 
-          _trap(
-            $self->_data_alias_loggable($session),
-            " can't watch $handle in mode $mode: ",
-            $self->_data_alias_loggable($hdl_rec->[HSS_SESSION]),
-            " is already watching it as $other_handle ($why)"
-          );
+          if ($session eq $watch_session) {
+            _die(
+              "A session was caught watching two different file handles that\n",
+              "reference the same file descriptor in the same mode ($mode).\n",
+              "This error is usually caused by a file descriptor leak.  The\n",
+              "most common cause is explicitly closing a filehandle without\n",
+              "first unregistering it from POE.\n",
+              "\n",
+              "Some possibly helpful information:\n",
+              "  Session    : ", $self->_data_alias_loggable($session), "\n",
+              "  Old handle : $other_handle (currently $why)\n",
+              "  New handle : $handle\n",
+              "\n",
+              "Please correct the program and try again.\n",
+            );
+          }
+          else {
+            _die(
+              "Two sessions were caught watching the same file descriptor\n",
+              "in the same mode ($mode).  This error is usually caused by\n",
+              "a file descriptor leak.  The most common cause is explicitly\n",
+              "closing a filehandle without first unregistering it from POE.\n",
+              "\n",
+              "Some possibly helpful information:\n",
+              "  Old session: ",
+              $self->_data_alias_loggable($hdl_rec->[HSS_SESSION]), "\n",
+              "  Old handle : $other_handle (currently $why)\n",
+              "  New session: ", $self->_data_alias_loggable($session), "\n",
+              "  New handle : $handle\n",
+              "\n",
+              "Please correct the program and try again.\n",
+            );
+          }
         }
       }
       _trap "internal inconsistency";
@@ -447,7 +382,6 @@ sub _data_handle_add {
     if ($kr_fno_rec->[FMO_REFCOUNT] == 1) {
       $self->loop_watch_filehandle($handle, $mode);
       $kr_fno_rec->[FMO_ST_ACTUAL]  = HS_RUNNING;
-      $kr_fno_rec->[FMO_ST_REQUEST] = HS_RUNNING;
     }
   }
 
@@ -476,6 +410,55 @@ sub _data_handle_add {
   }
 }
 
+### Condition a file handle so that it is ready for select et al
+sub _data_handle_condition {
+    my( $self, $handle ) = @_;
+
+    # For DOSISH systems like OS/2.  Wrapped in eval{} in case it's a
+    # tied handle that doesn't support binmode.
+    eval { binmode *$handle };
+
+    # Turn off blocking unless it's tied or a plain file.
+    unless (tied *$handle or -f $handle) {
+
+      unless (RUNNING_IN_HELL) {
+        if ($] >= 5.008) {
+          $handle->blocking(0);
+        }
+        else {
+          # Long, drawn out, POSIX way.
+          my $flags = fcntl($handle, F_GETFL, 0)
+            or _trap "fcntl($handle, F_GETFL, 0) fails: $!\n";
+          until (fcntl($handle, F_SETFL, $flags | O_NONBLOCK)) {
+            _trap(
+              "fcntl($handle [" . fileno($handle) . "], F_SETFL [" .
+              F_SETFL . "], $flags | O_NONBLOCK [" . O_NONBLOCK .
+              "]) fails: $!"
+            ) unless $! == EAGAIN or $! == EWOULDBLOCK;
+          }
+        }
+      }
+      else {
+        # Do it the Win32 way.
+        my $set_it = "1";
+
+        # 126 is FIONBIO (some docs say 0x7F << 16)
+        ioctl(
+          $handle,
+          0x80000000 | (4 << 16) | (ord('f') << 8) | 126,
+          \$set_it
+        ) or _trap(
+          "ioctl($handle, FIONBIO, $set_it) fails: errno " . ($!+0) . " = $!\n"
+        );
+      }
+    }
+
+    # Turn off buffering.
+    CORE::select((CORE::select($handle), $| = 1)[0]);
+}
+
+
+
 ### Remove a select from the kernel, and possibly trigger the
 ### session's destruction.
 
@@ -485,7 +468,7 @@ sub _data_handle_remove {
 
   # Make sure the handle is deregistered with the kernel.
 
-  if (exists $kr_filenos{$fd}) {
+  if (defined($fd) and exists($kr_filenos{$fd})) {
     my $kr_fileno  = $kr_filenos{$fd};
     my $kr_fno_rec = $kr_fileno->[$mode];
 
@@ -497,7 +480,10 @@ sub _data_handle_remove {
     ) {
 
       TRACE_FILES and
-        _warn "<fh> removing handle ($handle) fileno ($fd) mode ($mode)";
+        _warn(
+          "<fh> removing handle ($handle) fileno ($fd) mode ($mode) from " .
+          Carp::shortmess
+        );
 
       # Remove the handle from the kernel's session record.
 
@@ -507,9 +493,7 @@ sub _data_handle_remove {
       my $kill_session = $handle_rec->[HSS_SESSION];
       my $kill_event   = $handle_rec->[HSS_STATE];
 
-      # Remove any events destined for that handle.  Decrement
-      # FMO_EV_COUNT for each, because we've removed them.  This makes
-      # sense.
+      # Remove any events destined for that handle.
       my $my_select = sub {
         return 0 unless $_[0]->[EV_TYPE]    &  ET_SELECT;
         return 0 unless $_[0]->[EV_SESSION] == $kill_session;
@@ -523,22 +507,10 @@ sub _data_handle_remove {
         my ($time, $id, $event) = @$_;
         $self->_data_ev_refcount_dec( @$event[EV_SESSION, EV_SOURCE] );
 
-        TRACE_EVENTS and
-          _warn "<ev> removing select event $id ``$event->[EV_NAME]''";
-
-        $kr_fno_rec->[FMO_EV_COUNT]--;
-
-        if (TRACE_FILES) {
-          _warn(
-            "<fh> fileno $fd mode $mode event count went to ",
-            $kr_fno_rec->[FMO_EV_COUNT]
-          );
-        }
-
-        if (ASSERT_DATA) {
-          _trap "<dt> fileno $fd mode $mode event count went below zero"
-            if $kr_fno_rec->[FMO_EV_COUNT] < 0;
-        }
+        TRACE_EVENTS and _warn(
+          "<ev> removing select event $id ``$event->[EV_NAME]''" .
+          Carp::shortmess
+        );
       }
 
       # Decrement the handle's reference count.
@@ -556,7 +528,6 @@ sub _data_handle_remove {
       unless ($kr_fno_rec->[FMO_REFCOUNT]) {
         $self->loop_ignore_filehandle($handle, $mode);
         $kr_fno_rec->[FMO_ST_ACTUAL]  = HS_STOPPED;
-        $kr_fno_rec->[FMO_ST_REQUEST] = HS_STOPPED;
 
         # The session is not watching handles anymore.  Remove the
         # session entirely the fileno structure.
@@ -596,7 +567,7 @@ sub _data_handle_remove {
   }
 
   # SS_HANDLES - Remove the select from the session, assuming there is
-  # a session to remove it from.  -><- Key it on fileno?
+  # a session to remove it from.  TODO Key it on fileno?
 
   if (
     exists($kr_ses_to_handle{$session}) and
@@ -649,19 +620,12 @@ sub _data_handle_resume {
 
   if (TRACE_FILES) {
     _warn(
-      "<fh> resume test: fileno(" . fileno($handle) . ") mode($mode) " .
-      "count($kr_fno_rec->[FMO_EV_COUNT])"
+      "<fh> resume test: $handle fileno(" . fileno($handle) . ") mode($mode)"
     );
   }
 
-  # Resume the handle if there are no events for it.
-  unless ($kr_fno_rec->[FMO_EV_COUNT]) {
-    $self->loop_resume_filehandle($handle, $mode);
-    $kr_fno_rec->[FMO_ST_ACTUAL] = HS_RUNNING;
-  }
-
-  # Either way we set the handle's requested state to "running".
-  $kr_fno_rec->[FMO_ST_REQUEST] = HS_RUNNING;
+  $self->loop_resume_filehandle($handle, $mode);
+  $kr_fno_rec->[FMO_ST_ACTUAL] = HS_RUNNING;
 }
 
 ### Pause a filehandle.  If there are no events in the queue for this
@@ -676,19 +640,12 @@ sub _data_handle_pause {
 
   if (TRACE_FILES) {
     _warn(
-      "<fh> pause test: fileno(" . fileno($handle) . ") mode($mode) " .
-      "count($kr_fno_rec->[FMO_EV_COUNT])"
+      "<fh> pause test: $handle fileno(" . fileno($handle) . ") mode($mode)"
     );
   }
 
-  unless ($kr_fno_rec->[FMO_EV_COUNT]) {
-    $self->loop_pause_filehandle($handle, $mode);
-    $kr_fno_rec->[FMO_ST_ACTUAL] = HS_PAUSED;
-  }
-
-  # Correct the requested state so it matches the actual one.
-
-  $kr_fno_rec->[FMO_ST_REQUEST] = HS_PAUSED;
+  $self->loop_pause_filehandle($handle, $mode);
+  $kr_fno_rec->[FMO_ST_ACTUAL] = HS_PAUSED;
 }
 
 ### Return the number of active filehandles in the entire system.
@@ -724,7 +681,7 @@ sub _data_handle_clear_session {
   }
 }
 
-# -><- Testing accessors.  Maybe useful for introspection.  May need
+# TODO Testing accessors.  Maybe useful for introspection.  May need
 # modification before that.
 
 sub _data_handle_fno_refcounts {
@@ -737,24 +694,12 @@ sub _data_handle_fno_refcounts {
   )
 }
 
-sub _data_handle_fno_evcounts {
-  my ($self, $fd) = @_;
-  return(
-    $kr_filenos{$fd}->[FNO_MODE_RD]->[FMO_EV_COUNT],
-    $kr_filenos{$fd}->[FNO_MODE_WR]->[FMO_EV_COUNT],
-    $kr_filenos{$fd}->[FNO_MODE_EX]->[FMO_EV_COUNT],
-  )
-}
-
 sub _data_handle_fno_states {
   my ($self, $fd) = @_;
   return(
     $kr_filenos{$fd}->[FNO_MODE_RD]->[FMO_ST_ACTUAL],
-    $kr_filenos{$fd}->[FNO_MODE_RD]->[FMO_ST_REQUEST],
     $kr_filenos{$fd}->[FNO_MODE_WR]->[FMO_ST_ACTUAL],
-    $kr_filenos{$fd}->[FNO_MODE_WR]->[FMO_ST_REQUEST],
     $kr_filenos{$fd}->[FNO_MODE_EX]->[FMO_ST_ACTUAL],
-    $kr_filenos{$fd}->[FNO_MODE_EX]->[FMO_ST_REQUEST],
   );
 }
 
@@ -779,30 +724,41 @@ __END__
 
 =head1 NAME
 
-POE::Resources::FileHandles - manage file handles on behalf of POE::Kernel
+POE::Resource::FileHandles - internal filehandle manager for POE::Kernel
 
 =head1 SYNOPSIS
 
-Used internally by POE::Kernel.  Better documentation will be
-forthcoming.
+There is no public API.
 
 =head1 DESCRIPTION
 
-This module encapsulates low-level file handle management for
-POE::Kernel.  It provides accessors to its data structures that
-POE::Kernel uses internally.  This module has no public interface.
-Move along.
+POE::Resource::FileHandles is a mix-in class for POE::Kernel.  It
+provides the low-level features to manage filehandles.  It is used
+internally by POE::Kernel, so it has no public interface.
 
 =head1 SEE ALSO
 
-See L<POE::Kernel> for documentation on file handles and selects.
+See L<POE::Kernel/I/O Watchers (Selects)> for the public file watcher
+API.
+
+See L<POE::Kernel/Resources> for for public information about POE
+resources.
+
+See L<POE::Resource> for general discussion about resources and the
+classes that manage them.
 
 =head1 BUGS
 
-Probably.
+POE watches I/O based on filehandles rather than file descriptors,
+which means there can be clashes between its API and an underlying
+descriptor-based event loop.  This is usually not a problem, but it
+may require a work-around in certain edge cases.
 
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
